package PrimeTime::Reporter ;
use Text::Histogram  ;
use POSIX qw(ceil floor) ;
use Getopt::Long qw(GetOptionsFromArray);


use strict;
use warnings;

sub new {
  my $self=shift;
  my $class=ref($self) || $self;
  my %h;
  $h{size}=0 ;
  return bless {%h}, $class;
}

# The following subroutine to actually parse the PT report has been borrowed from PrimeTime::Report whose original author is yorkwu (yorkwuo@gmail.com)
sub read_file {
  my $self=shift;
  my $infile=shift;


  unless (-e $infile) {
    print "$infile does not exist\n" ;
    return ;
  }
  if ($infile =~ /\.gz$/) {
    open (FIN, "gunzip -c $infile |");
  } else {
    open (FIN, $infile);
  }

  #open FIN, "<$infile" or die "$!";
  undef $/;
  my $file = <FIN>;
  $/='\n';
  close FIN;

  my @path = $file =~ m/(Startpoint.*?slack.*?)$/gsm;
  my $count = $self->{size};
  my $startpoint;
  my $endpoint;
  my $path_group;
  my $path_type;
  my $uncertainty;
  my $slack;
  my $clock_domain;
  my @part;
  my $clock_source_rise_time;
  my $clock_hit_source_FF;
  my $clock_hit_capture_FF;
  my $clock_capture_rise_time;
  my $clock_period;
  my $clock_latency_source;
  my $clock_latency_capture;
  my $skew;
  my @part_source;
  my $clock_path_capture;

  foreach my $eachpath (@path){
    $startpoint   = "N/A" if(!(($startpoint) = $eachpath =~ m/Startpoint: (.*?)$/m));
    $endpoint     = "N/A" if(!(($endpoint) = $eachpath =~ m/Endpoint: (.*?)$/m));
    $path_group   = "N/A" if(!(($path_group) = $eachpath =~ m/Path Group: (.*?)$/m));
    $path_type    = "N/A" if(!(($path_type) = $eachpath =~ m/Path Type: (.*?)$/m));
    $uncertainty  = "N/A" if(!(($uncertainty) = $eachpath =~ m/inter-clock uncertainty[ ]+(\S*?) /m));
    $slack        = "N/A" if(!(($slack) = $eachpath =~ m/^[ ]+slack \(.+\)[ ]+(\S*?)$/sm));
    $clock_domain = "N/A" if(!(($clock_domain) = $eachpath =~ m/ clock (\S*?) \(rise edge\)/m));
    @part = split(/^\s*$/sm,$eachpath);
    @part_source = split(m/($startpoint.*)/sm, $part[1]);
    ($clock_path_capture) = $part[2] =~ m/(.*$endpoint.*? [rf])/sm;
    $clock_source_rise_time = "0" if(!(($clock_source_rise_time) = $part[1] =~ m/ clock \S*? \(rise edge\)[ ]+(\S*?) /m));
    $clock_hit_source_FF    = "0" if(!(($clock_hit_source_FF) = $part[1] =~ m/$startpoint.*?(\S*?) [rf]/sm));
    $clock_capture_rise_time= "0" if(!(($clock_capture_rise_time) = $part[2] =~ m/ clock \S*? \(rise edge\)[ ]+(\S*?) /m));
    $clock_period           = $clock_capture_rise_time - $clock_source_rise_time;
    $clock_hit_capture_FF    = "0" if(!(($clock_hit_capture_FF) = $part[2] =~ m/$endpoint.*?(\S*?) [rf]/sm));

    if(!(($clock_latency_source) = $part[1] =~ m/ clock network delay \(\w+\)[ ]+(\S*?) /m)){
      $clock_latency_source = $clock_hit_source_FF - $clock_source_rise_time;
    }
    if(!(($clock_latency_capture) = $part[2] =~ m/ clock network delay \(\w+\)[ ]+(\S*?) /m)){
      $clock_latency_capture = $clock_hit_capture_FF - $clock_capture_rise_time;
    }
    $skew = sprintf("%.4f",$clock_latency_capture - $clock_latency_source);

    $self->{paths}->{$count} = {
      raw=> $eachpath,
      startpoint => $startpoint,
      endpoint => $endpoint,
      path_group => $path_group,
      path_type => $path_type,
      clock_domain => $clock_domain,
      clock_period => $clock_period,
      uncertainty => $uncertainty,
      clock_path_source => $part_source[0],
      clock_path_capture => $clock_path_capture,
      clock_source_rise_time => $clock_source_rise_time,
      clock_hit_source_FF => $clock_hit_source_FF,
      clock_latency_source => $clock_latency_source,
      clock_latency_capture => $clock_latency_capture,
      skew => $skew,
      slack => $slack,
      start_part => $part_source[1],
      end_part=>$part[2],
    };
    $count++;
  }
  my $new_paths=$count-$self->{size} ;
  $self->{size}=$count ;
  return $new_paths ;
}

sub slack_histo {
  my $slack_data ; #Reference to slack array
  my $self=shift ;

  my $size=$self->{size} ;

  my $step = 50.0 ;
  $step = shift if (@_) ;

  unless ($step > 0) {
    print "Need a positive step value\n" ;
    return ;
  }
  for (my $i=0 ; $i < $size ; $i++) {
      push @$slack_data, $self->{paths}->{$i}->{slack} ;
  }

  _histo ($slack_data, $step) ;
}


sub _histo
{
  my ($list_ref, $step) = @_ ;

  # This calculates the frequencies for all available bins in the data set
  my %histogram;
  $histogram{ceil($_/$step+1)-1}++ for @$list_ref ;

  my $max;
  my $min;
  my $max_freq=0;

  # Calculate min and max
  while ( my ($key, $value) = each(%histogram) )
  {
    $max = $key if !defined($max) || $key > $max;
    $min = $key if !defined($min) || $key < $min;
    $max_freq = $value if $value > $max_freq ;
  }


  for (my $i = $min; $i <= $max; $i++)
  {
    my $bin       = sprintf("%8s to %8s", $i*$step-$step, $i*$step);
    my $frequency = $histogram{$i} || 0;

    my $hash_factor = ceil($max_freq/60) ;
    my $freq_symbol = "#" x (ceil($frequency/$hash_factor));

    printf "%20s : %-60s : %8d\n", $bin, $freq_symbol, $frequency ;
  }

}

sub filter {
  my $self=shift ;

  my $size=$self->{size} ;
  
  #my $filtered_rpt=$self->new ;
  my $dest=shift ;
  my $j=0 ;
  for (my $i=0 ; $i < $size ; $i++) {
      if ($self->match_spec($i, @_)) {
          $dest->{paths}->{$j}=$self->{paths}->{$i} ;
          $j++ ;
      }
  }
  $dest->{size}+=$j ;

  return $j ;
}

sub match_spec {
  my $self=shift ;
  my $i=shift ;
  my $path=$self->{paths}->{$i}->{raw} ;

  my @exclude ;
  my @include ;

  GetOptionsFromArray(\@_, "e=s" => \@exclude, "r=s"=>\@include) ;

  my $criteria_met=1 ;
  
  foreach (@include) {
    $criteria_met=0 if ($path !~ /$_/) ;
  }

  foreach (@exclude) {
      $criteria_met=0 if ($path =~ /$_/) ;
  }
  return $criteria_met ;
}

sub write {
  my $self=shift ;
  my $output_file=shift ;

  open (OUTFILE, ">$output_file") ;
  for (my $i=0 ; $i < $self->{size} ; $i++) {
    print OUTFILE "  $self->{paths}->{$i}->{raw}\n\n" ;
  }
  close OUTFILE ;
}

sub write_with_skew {
  my $self=shift ;
  my $output_file=shift ;

  open (OUTFILE, ">$output_file") ;
  for (my $i=0 ; $i < $self->{size} ; $i++) {
    print OUTFILE "  $self->{paths}->{$i}->{raw}" ;
    print OUTFILE "\n  Clock skew = $self->{paths}->{$i}->{skew}\n\n" ;
  }
  close OUTFILE ;
}

sub less {
  my $self=shift ;
  
  local $SIG{PIPE} = "IGNORE" ;
  open (OUTFILE, "|less") ;
  for (my $i=0 ; $i < $self->{size} ; $i++) {
    print OUTFILE "  $self->{paths}->{$i}->{raw}\n\n" ;
  }
  close OUTFILE ;
}

sub qor {
  my $self=shift ;
  my $size=$self->{size} ;
  my $wns ;
  my $tns=0 ;
  my $viol_count=0 ;

  for (my $i=0 ; $i < $size ; $i++) {
    my $slack=$self->{paths}->{$i}->{slack} ;
    $wns = $slack if !defined($wns) || $slack < $wns ;
    $tns+=$wns if $wns<0 ;
    $viol_count++ if $wns<0 ;
  }

  print "WNS=$wns, total violations=$viol_count, TNS=$tns\n" ;
}

1 ;

#vim:ts=2
