package PrimeTime::RptShell ;
use strict;
use warnings;

use PrimeTime::Reporter ;
use base qw(Term::Shell) ;

sub run_help {
    print "I am the primetime reporter. Currently I support the following commands:\n" ;
    print "help\n" ;
}

sub prompt_str { "rpt_shell> " }

sub run_load {
    my $self=shift ;
    if (scalar @_ < 2) {
        print "Usage: load <rpt_id> <rpt_file>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    my $file=shift ;
    $self->{rpts}->{$rpt_id}=new PrimeTime::Reporter if (!defined($self->{rpts}->{$rpt_id})) ;
    my $read_paths=$self->{rpts}->{$rpt_id}->read_file($file) ;
    print "$read_paths paths read, bringing the size of rpt $rpt_id to $self->{rpts}->{$rpt_id}->{size}\n" ;
}

sub run_histo {
    my $self=shift ;
    if (scalar @_ < 2) {
        print "Usage: histo <rpt_id> <bin_width>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    if (!defined($self->{rpts}->{$rpt_id})) {
        print "No such report loaded: $rpt_id\n" ;
        return ;
    }
    $self->{rpts}->{$rpt_id}->slack_histo(@_) ;
}

sub run_filter {
    my $self = shift ;

    if (scalar @_ == 0) {
        print "Usage: filter <source_rpt> <dest_rpt> <filter_options>\n" ;
        print "dest_rpt need not exist\nExample:\n" ;
        print "filter 0 f -e usblk_smu\n" ;
        return ;
    }
    my $src=shift ;
    my $dest_rpt=$_[0] ;
    $self->{rpts}->{$dest_rpt}=new PrimeTime::Reporter if (!defined($self->{rpts}->{$dest_rpt})) ;
    my $added_paths =$self->{rpts}->{$src}->filter($self->{rpts}->{$dest_rpt}, @_) ;
    print "Added $added_paths paths to $dest_rpt, whose size is now $self->{rpts}->{$dest_rpt}->{size}\n" ;
}

sub run_list {
    my $self=shift ;

    my @rpt=keys %{$self->{rpts}} ;
    print "@rpt\n" ;
}

sub run_write {
    my $self=shift ;
    if (scalar @_ < 2) {
        print "Usage: write <rpt_id> <dest_file>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    if (!defined($self->{rpts}->{$rpt_id})) {
        print "Unknown rpt $rpt_id, use list to get all the loaded rpts\n" ;
        return ;
    }
    my $output_file=shift ;
    $self->{rpts}->{$rpt_id}->write($output_file) ;
}

sub run_write_with_skew {
    my $self=shift ;
    if (scalar @_ < 2) {
        print "Usage: write <rpt_id> <dest_file>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    if (!defined($self->{rpts}->{$rpt_id})) {
        print "Unknown rpt $rpt_id, use list to get all the loaded rpts\n" ;
        return ;
    }
    my $output_file=shift ;
    $self->{rpts}->{$rpt_id}->write_with_skew($output_file) ;
}

sub run_less {
    my $self=shift ;
    unless (scalar @_) {
        print "Usage: less <rpt_id>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    if (!defined($self->{rpts}->{$rpt_id})) {
        print "Unknown rpt $rpt_id, use list to get all the loaded rpts\n" ;
        return ;
    }

    $self->{rpts}->{$rpt_id}->less ;
}

sub run_qor {
    my $self = shift ;
    unless (scalar @_) {
        print "Usage: qor <rpt_id>\n" ;
        return ;
    }
    my $rpt_id=shift ;
    if (!defined($self->{rpts}->{$rpt_id})) {
        print "Unknown rpt $rpt_id, use list to get all the loaded rpts\n" ;
        return ;
    }

    $self->{rpts}->{$rpt_id}->qor ;
}

1 ;
#vim:ts=4
